# THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
# KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
# PARTICULAR PURPOSE.

import pytest
import numpy as np
import json

from typhoon.api.schematic_editor import model
import typhoon.test.capture as cap
import typhoon.test.reporting.messages as msg
import typhoon.api.hil as hil
from typhoon.api.schematic_editor import SchematicAPI
import typhoon.test.rms as rms
from typhoon.types.timedelta import Timedelta as td
import typhoon.test.signals as signals


# Path to model file and path to compiled model file
model = SchematicAPI()
MODEL_PATH = 'double IM test.tse'
COMPILED_MODEL_PATH = model.get_compiled_model_file(MODEL_PATH)
component_name = 'Machine Electrical Parameter Estimator'
component_name2 = 'test machine'
machine_list = []

#loading machines from .json file for testing
JSON_FILE_PATH = 'machine_parameters.json'
TEST_TYPE = 'Three Phase Squirrel Cage Induction Machine'
with open(JSON_FILE_PATH, 'r') as file:
    for line in file:
        machine = json.loads(line)
        if machine['Type'] == TEST_TYPE:
            machine_list.append(machine)

#loading model
@pytest.fixture(scope="module")
def setup():
    model.load(MODEL_PATH)
    
#setting values into model, model compilation
@pytest.fixture(params=machine_list)
def new_cycle(request):
    
    settings = request.param
    component = model.get_item(name=component_name)
    
    model.set_property_value(model.prop(component, 'Pout'), settings['Poutn'])
    model.set_property_value(model.prop(component, 'V'), settings['Vn'])
    model.set_property_value(model.prop(component, 'p'), settings['p'])
    model.set_property_value(model.prop(component, 'eta'), settings['eta'][2])
    model.set_property_value(model.prop(component, 'pf'), settings['pf'][2])
    model.set_property_value(model.prop(component, 'f'), settings['f'])
    model.set_property_value(model.prop(component, 'n_r'), settings['nn'])
    model.set_property_value(model.prop(component, 'I_r'), settings['I_r'])
    model.set_property_value(model.prop(component, 'I0'), settings['I0'])
    model.set_property_value(model.prop(component, 'T_r'), settings['Tn'])
    model.set_property_value(model.prop(component, 'T0'), settings['T0'])
    model.set_property_value(model.prop(component, 'Tmax'), settings['Tmax'])
    
    model.set_property_value(model.prop(component, 'type'), 'Three Phase Double Cage Induction Machine')
    model.set_property_value(model.prop(component, 'component'), component_name2)
    model.set_property_value(model.prop(component, 'r_snb_param'), True)
    model._click_button(model.prop(component, 'estimate'))
    model._click_button(model.prop(component, 'parametrize_component'))
    
    component2 = model.get_item(name = component_name2)
    model.set_property_value(model.prop(component2, 'pms'), settings['p'])
    model.set_property_value(model.prop(component2, 'Jm'), 3*settings['J'])
    
    model.set_property_value(model.prop(model.get_item(name = 'Nominal torque'), 'value'), settings['Tn'])
    
    model.compile(conditional_compile=False)                                                #needs to be False to change the parameters value
    hil.load_model(file = COMPILED_MODEL_PATH, vhil_device=False)
    hil.set_scada_input_value(scadaInputName='torque', value=0.0)
    hil.set_source_sine_waveform('Voltage source', settings['Vn']/1.73, settings['f']) 
    hil.set_contactor('S1', True, True)
    hil.start_simulation()
    
    yield request.param
    
    hil.stop_simulation()

#test for validation of evaluated parameters
def test_machine(setup, new_cycle):
    
    #defining variables with catalogue values
    n_r = new_cycle['nn']                   #rated speed
    f = new_cycle['f']                      #rated frequency
    I_r = new_cycle['I_r']                  #rated current
    Tmax = new_cycle['Tmax']                #breakdown torque
    Tn = new_cycle['Tn']                    #rated torque
    
    FILE_NAME = 'Capture\\'+str(new_cycle['Poutn'])+'W_'+str(new_cycle['nn'])+'min-1.csv'
    #capturing the signals
    cap.start_capture(duration=10, rate=1e4, signals=['n_', 'T_', 'Ia', 'Ib', 'Ic'], fileName = FILE_NAME)
    #getting the machine in nominal state
    hil.wait_sec(5)
    hil.set_scada_input_value(scadaInputName='torque', value=1)
    hil.wait_sec(3)
    

    df = cap.get_capture_results(wait_capture=True)
    
    #testing measured speed to be +-3% of rated
    signals.assert_is_constant(signal=df['n_'], at_value=(0.97*n_r, 1.03*n_r), strictness=0.95, during=(9, 9.9))
    
    #testing measured current to be +-20% of rated
    with pytest.assume:
        #calculation of current RMS
        df['Ia_rms'] = rms.window_rms(signal = df['Ia'], window_length = 1/f)
        df['Ib_rms'] = rms.window_rms(signal = df['Ib'], window_length = 1/f)
        df['Ic_rms'] = rms.window_rms(signal = df['Ic'], window_length = 1/f)
        time_eval_start = df.index[-1] - td(10/f)
        RMSmean = (df['Ia_rms'][time_eval_start:].mean() + df['Ib_rms'][time_eval_start:].mean() +
                   df['Ic_rms'][time_eval_start:].mean()) / 3
        
        #test condition
        current_test = 0.8*I_r < RMSmean < 1.2*I_r
        #printing test results
        if current_test:
            msg.report_message(f"Measured current is in +-20% of nominal current.")
        else:
            msg.report_message(f"Measured current is not in +-20% of nominal current.")
        msg.report_message(f"Measured current RMS = {round(RMSmean/I_r*100, 2)}% of nominal current.")
        assert current_test
    
    
    #testing breakdown torque to be greater than 85% of catalogue one
    with pytest.assume:
        #capturing signals
        cap_time = (Tmax - 1) * 2 + 0.5
        hil.set_scada_input_value(scadaInputName='torque', value=Tmax)
        cap.start_capture(duration=cap_time, rate=1e4, signals=['n_', 'T_'])
        df = cap.get_capture_results(wait_capture=True)
        
        #test condition
        torque_test = max(df['T_']) / (0.85*Tmax*Tn) > 1
        #printing test results
        if torque_test:
            msg.report_message(f"Estimated maximum torque is greater than 85% of cataloague maximum torque")
        else:
            msg.report_message(f"Estimated maximum torque is not greater than 85% of catalogue maximum torque")
        msg.report_message(f"T_est = {round(max(df['T_'])/(Tmax*Tn)*100, 2)}% of Tmax")
        assert torque_test
    
    #starting simulation again to test snubber
    hil.stop_simulation()
    hil.set_scada_input_value(scadaInputName='torque', value=0)
    hil.start_simulation()
    hil.wait_sec(4)
    hil.set_scada_input_value(scadaInputName='torque', value=1)
    hil.wait_sec(2)
    
    ao_flag = False
    for i in range(10):
        hil.set_contactor('S1', True, False)
        ao_flag = hil.get_flag_status(hil.FL_ARITHMETIC_OVERFLOW)
        hil.set_contactor('S1', True, True)
        hil.wait_msec(103)
        if ao_flag:
            break
    
    msg.report_message(f"Aritmetic overflow flag is {ao_flag}.")
    assert (not ao_flag)









