# THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
# KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
# PARTICULAR PURPOSE.

import pytest
import numpy as np
import json

from typhoon.api.schematic_editor import model
import typhoon.test.signals as signals
import typhoon.test.capture as cap
import typhoon.test.reporting.messages as msg
import typhoon.api.hil as hil
from typhoon.api.schematic_editor import SchematicAPI
import typhoon.test.rms as rms
from typhoon.types.timedelta import Timedelta as td


# Path to model file and path to compiled model file
model = SchematicAPI()
MODEL_PATH = "IM test.tse"
COMPILED_MODEL_PATH = model.get_compiled_model_file(MODEL_PATH)
component_name = 'Machine Electrical Parameter Estimator'
component_name2 = 'test machine'
machine_list = []

#loading machines from .json file for testing
JSON_FILE_PATH = 'machine_parameters.json'
TEST_TYPE = 'Three Phase Squirrel Cage Induction Machine'
with open(JSON_FILE_PATH, 'r') as file:
    for line in file:
        machine = json.loads(line)
        if machine['Type'] == TEST_TYPE:
            machine_list.append(machine)

#loading model
@pytest.fixture(scope="module")
def setup():
    model.load(MODEL_PATH)
    
#setting values into model, model compilation
@pytest.fixture(params=machine_list)
def new_cycle(request):
    
    settings = request.param
    component = model.get_item(name=component_name)
    
    model.set_property_value(model.prop(component, 'Pout'), settings['Poutn'])
    model.set_property_value(model.prop(component, 'V'), settings['Vn'])
    model.set_property_value(model.prop(component, 'p'), settings['p'])
    model.set_property_value(model.prop(component, 'eta'), settings['eta'])
    model.set_property_value(model.prop(component, 'pf'), settings['pf'])
    model.set_property_value(model.prop(component, 'f'), settings['f'])
    model.set_property_value(model.prop(component, 'n_r'), settings['nn'])
    model.set_property_value(model.prop(component, 'a'), settings['a'])
    
    model.set_property_value(model.prop(component, 'type'), 'Three Phase Squirrel Cage Induction Machine')
    model.set_property_value(model.prop(component, 'component'), component_name2)
    model.set_property_value(model.prop(component, 'r_snb_param'), True)
    model._click_button(model.prop(component, 'estimate'))
    model._click_button(model.prop(component, 'parametrize_component'))
    
    Prot = model.get_property_value(model.prop(component, 'Prot'))
    
    component2 = model.get_item(name = component_name2)
    model.set_property_value(model.prop(component2, 'pms'), settings['p'])
    model.set_property_value(model.prop(component2, 'Jm'), settings['J'])
    
    model.set_property_value(model.prop(model.get_item(name = 'Prot'), 'value'), Prot)
    model.set_property_value(model.prop(model.get_item(name = 'Nominal torque'), 'value'), settings['Tn'])
    
    model.compile(conditional_compile=False)                                               #needs to be False to change the parameters value
    hil.load_model(file = COMPILED_MODEL_PATH, vhil_device=True)
    hil.set_scada_input_value(scadaInputName='torque', value=0.0)
    hil.set_source_sine_waveform('Voltage source', settings['Vn']/1.73, settings['f'])
    hil.set_contactor('S1', True, True)
    hil.start_simulation()
    
    yield request.param
    
    hil.stop_simulation()

#test for validation of evaluated parameters
def test_machine(setup, new_cycle):
    
    #defining variables with catalogue values
    n_r = new_cycle['nn']                   #rated speed
    f = new_cycle['f']                      #rated frequency
    I_r = new_cycle['I_r']                  #rated current
    Tmax = new_cycle['Tmax']                #breakdown torque
    Tn = new_cycle['Tn']                    #rated torque
    eta_cat = new_cycle['eta'][2]           #efficency coeficent
    pf_cat = new_cycle['pf'][2]             #power factor
    FILE_NAME = 'Capture\\'+str(new_cycle['Poutn'])+'W_'+str(new_cycle['nn'])+'min-1.csv'
    #getting the machine in nominal state
    hil.wait_sec(3)
    hil.set_scada_input_value(scadaInputName='torque', value=1)
    hil.wait_sec(2.5)
    
    #capturing the signals
    cap.start_capture(duration=1, rate=1e4, signals=['n', 'Tel', 'Iaa', 'Ibb', 'Icc', 'eta', 'pf'], fileName = FILE_NAME)
    df = cap.get_capture_results(wait_capture=True)
    
    #testing measured speed to be +-3% of rated
    signals.assert_is_constant(signal=df['n'], at_value=(0.97*n_r, 1.3*n_r), strictness=0.95)
    
    #testing measured current to be +-10% of rated
    with pytest.assume:
        #calculation of current RMS
        df['Ia_rms'] = rms.window_rms(signal = df['Iaa'], window_length = 1/f)
        df['Ib_rms'] = rms.window_rms(signal = df['Ibb'], window_length = 1/f)
        df['Ic_rms'] = rms.window_rms(signal = df['Icc'], window_length = 1/f)
        time_eval_start = df.index[-1] - td(10/f)
        RMSmean = (df['Ia_rms'][time_eval_start:].mean() + df['Ib_rms'][time_eval_start:].mean() +
                   df['Ic_rms'][time_eval_start:].mean()) / 3
        
        #test condition
        current_test = 0.9*I_r < RMSmean < 1.1*I_r
        #printing test results
        if current_test:
            msg.report_message(f"Measured current is in +-10% of nominal current.")
        else:
            msg.report_message(f"Measured current is not in +-10% of nominal current.")
        msg.report_message(f"Measured current RMS = {round(RMSmean/I_r*100, 2)}% of nominal current.")
        assert current_test
    
    #testing efficency to be +-0.02 of rated
    with pytest.assume:
        msg.report_message(f"eta/eta_cat = {df['eta'].mean()}/{eta_cat} = {df['eta'].mean()/eta_cat*100}%")
        signals.assert_is_constant(signal=df['eta'], at_value=(eta_cat-0.02, eta_cat + 0.02), strictness=0.95)
        
    #testing power factor to be +-0.05 of rated
    with pytest.assume:
        msg.report_message(f"pf/pf_cat = {df['pf'].mean()}/{pf_cat} = {df['pf'].mean()/pf_cat*100}%")
        signals.assert_is_constant(signal=df['pf'], at_value=(pf_cat-0.05, pf_cat + 0.05), strictness=0.95)
    
    #testing breakdown torque to be greater than 85% of catalogue one
    with pytest.assume:
        #capturing signals
        cap_time = (Tmax - 1) * 2 + 0.5
        hil.set_scada_input_value(scadaInputName='torque', value=Tmax)
        cap.start_capture(duration=cap_time, rate=1e4, signals=['n', 'Tel'])
        df = cap.get_capture_results(wait_capture=True)
        
        #test condition
        torque_test = max(df['Tel']) / (0.85*Tmax*Tn) > 1
        #printing test results
        if torque_test:
            msg.report_message(f"Estimated maximum torque is greater than 85% of cataloague maximum torque.")
        else:
            msg.report_message(f"Estimated maximum torque is not greater than 85% of catalogue maximum torque.")
        msg.report_message(f"T_est = {round(max(df['Tel'])/(Tmax*Tn)*100, 2)}% of Tmax.")
        assert torque_test
        
    #starting simulaion again to test snubber
    hil.stop_simulation()
    hil.set_scada_input_value(scadaInputName='torque', value=0)
    hil.start_simulation()
    hil.wait_sec(3)
    hil.set_scada_input_value(scadaInputName='torque', value=1)
    hil.wait_sec(2.5)
    
    ao_flag = False
    for i in range(10):
        hil.set_contactor('S1', True, False)
        ao_flag = hil.get_flag_status(hil.FL_ARITHMETIC_OVERFLOW)
        hil.set_contactor('S1', True, True)
        hil.wait_msec(83)
        if ao_flag:
            break
    
    msg.report_message(f"Aritmetic overflow flag is {ao_flag}.")
    assert (not ao_flag)
    
        









