# THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
# KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
# PARTICULAR PURPOSE.

import pytest
import numpy as np
import json

from typhoon.api.schematic_editor import model
import typhoon.test.signals as signals
import typhoon.test.capture as cap
import typhoon.test.reporting.messages as msg
import typhoon.api.hil as hil
from typhoon.api.schematic_editor import SchematicAPI
import typhoon.test.harmonic as harm
import typhoon.test.rms as rms
from typhoon.types.timedelta import Timedelta as td


# Path to model file and path to compiled model file
model = SchematicAPI()
MODEL_PATH = "SM test.tse"
COMPILED_MODEL_PATH = model.get_compiled_model_file(MODEL_PATH)
component_name = 'Machine Electrical Parameter Estimator'
machine_name = 'test machine'
machine_list = []

#loading machines from .json file for testing
JSON_FILE_PATH = 'machine_parameters.json'
TEST_TYPE = 'Three Phase Wound Rotor Synchronous Machine'
with open(JSON_FILE_PATH, 'r') as file:
    for line in file:
        machine = json.loads(line)
        if machine['Type'] == TEST_TYPE:
            machine_list.append(machine)

#loading model
@pytest.fixture(scope="module")
def setup():
    model.load(MODEL_PATH)
    
#setting values into model, model compilation
@pytest.fixture(params=machine_list[:3])
def new_cycle(request):
    
    settings = request.param
    component = model.get_item(name=component_name)
    
    model.set_property_value(model.prop(component, 'Pout'), settings['Poutn'])
    model.set_property_value(model.prop(component, 'V'), settings['Vn'])
    model.set_property_value(model.prop(component, 'eta'), settings['eta'])
    model.set_property_value(model.prop(component, 'pf'), settings['pf'])
    model.set_property_value(model.prop(component, 'f'), settings['f'])
    model.set_property_value(model.prop(component, 'n_r'), settings['nn'])
    
    model.set_property_value(model.prop(component, 'type'), 'Three Phase Wound Rotor Synchronous Machine')
    model.set_property_value(model.prop(component, 'component'), machine_name)
    model.set_property_value(model.prop(component, 'r_snb_param'), True)
    model._click_button(model.prop(component, 'estimate'))
    model._click_button(model.prop(component, 'parametrize_component'))
    
    model.set_property_value(model.prop(model.get_item(name = machine_name), 'Jm'), 2*settings['J'])
    
    model.set_property_value(model.prop(model.get_item(name = 'Nominal torque'), 'value'), settings['Tn'])
    model.set_property_value(model.prop(model.get_item(name = 'Nominal power factor'), 'value'), settings['pf'])
    
    model.compile(conditional_compile=False)                                                #needs to be False to change the parameters value
    hil.load_model(file = COMPILED_MODEL_PATH, vhil_device=False)
    hil.set_scada_input_value(scadaInputName='torque', value=0.1)
    hil.set_scada_input_value(scadaInputName='Vf', value=10)
    hil.set_scada_input_value(scadaInputName='Vf_control', value=0)
    hil.set_machine_initial_speed(name=machine_name, speed=settings['nn']/60*2*3.14)
    hil.set_source_sine_waveform('Voltage source', rms = settings['Vn']/1.73, frequency = settings['f'])
    hil.set_contactor('S1', True, True)
    hil.start_simulation()
    
    yield request.param
    
    hil.stop_simulation()

#test for validation of evaluated parameters
def test_machine(setup, new_cycle):
    
    #defining variables with catalogue values
    n_r = new_cycle['nn']                   #rated speed
    Tn = new_cycle['Tn']
    I_r = new_cycle['I_r']
    f = new_cycle['f']
    FILE_NAME = 'Capture\\'+str(new_cycle['Poutn'])+'W_'+str(new_cycle['nn'])+'min-1.csv'
    #getting the machine in nominal state
    cap.start_capture(duration=15, rate=1e3, signals=['n_', 'T_', 'Ia', 'Ib', 'Ic', 'power factor', 'Vf_'], fileName=FILE_NAME)
    hil.wait_sec(3)
    hil.set_scada_input_value(scadaInputName='torque', value=1)
    hil.set_scada_input_value(scadaInputName='Vf', value=30)
    hil.wait_sec(2)
    hil.set_scada_input_value(scadaInputName='Vf_control', value=1)
    hil.wait_sec(5)
    
    #capturing the signals
    df = cap.get_capture_results(wait_capture=True)
    
    #testing measured speed to be +-2rpm of rated
    signals.assert_is_constant(signal=df['n_'], at_value=(n_r - 2, n_r + 2), strictness=1, during=(13.9,14.9))
    
    #testing torque to be +-2% of rated
    with pytest.assume:
        signals.assert_is_constant(signal=df['T_'], at_value=(0.98*Tn, 1.02*Tn), strictness=1, during=(13.9,14.9))
        
    #testing measured current to be +-5% of rated
    with pytest.assume:
        #calculation of current RMS
        df['Ia_rms'] = rms.window_rms(signal = df['Ia'], window_length = 1/f)
        df['Ib_rms'] = rms.window_rms(signal = df['Ib'], window_length = 1/f)
        df['Ic_rms'] = rms.window_rms(signal = df['Ic'], window_length = 1/f)
        time_eval_start = df.index[-1] - td(10/f)
        RMSmean = (df['Ia_rms'][time_eval_start:].mean() + df['Ib_rms'][time_eval_start:].mean() +
                   df['Ic_rms'][time_eval_start:].mean()) / 3
        
        #test condition
        current_test = 0.95*I_r < RMSmean < 1.05*I_r
        #printing test results
        if current_test:
            msg.report_message(f"Measured current is in +-5% of nominal current.")
        else:
            msg.report_message(f"Measured current is not in +-5% of nominal current.")
        msg.report_message(f"Measured current RMS = {round(RMSmean/I_r*100, 2)}% of nominal current.")
        assert current_test
        

    #snubber test
    ao_flag = False
    for i in range(10):
        hil.set_contactor('S1', True, False)
        ao_flag = hil.get_flag_status(hil.FL_ARITHMETIC_OVERFLOW)
        hil.set_contactor('S1', True, True)
        hil.wait_msec(157)
        if ao_flag:
            break

    msg.report_message(f"Aritmetic overflow flag is {ao_flag}.")
    assert (not ao_flag)








